package com.delfines.spring.controller;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.nio.ByteBuffer;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.delfines.business.Util;

/**
 * Conversion del xml ubl 2.0 a xml ubl 2.1 para micros 9700 HMS
 * @author master
 */
public class CorrectToUBL21SUNAT {

	public static final String xmlPathFrom = "\\\\10.0.0.127\\c$\\Program Files (x86)\\Signature\\xDoc PE 2.1\\Stores\\areas\\2DISPATCH-DOC\\toprint\\";
	public static final String xmlPathTo ="\\\\10.0.0.127\\c$\\Program Files (x86)\\Signature\\xDoc PE 2.1\\Stores\\areas\\2DISPATCH-DOC\\instances\\01\\outXML\\";
	
	//public static final String xmlPathFrom = "D:\\claudio\\25102021\\SUNAT\\";
	//public static final String xmlPathTo ="D:\\claudio\\25102021\\SUNAT\\";
	
    public static void main(String argv[]) {
    	
    	String filename="01_2021-10-17 14-15-36.xml";
    	String fileFrom = xmlPathFrom+filename;
    	String fileTo = xmlPathTo+filename.replaceAll(" ","-").replaceAll(".xml", "OK.xml");
    	/*
    	CorrectToUBL21SUNAT t =new CorrectToUBL21SUNAT();
    	t.convert(fileFrom,fileTo,"2021-10-18");
    	*/

    	try {
	        CorrectToUBL21SUNAT t =new CorrectToUBL21SUNAT();
	        Document doc=t.convertDescuento(fileFrom);
	        String text=t.getStringFromDocument(doc);
	        
    		ByteBuffer buffer = StandardCharsets.UTF_8.encode(text); 
    		String textEncoded = StandardCharsets.UTF_8.decode(buffer).toString();
    		
    		
    		 BufferedWriter writer = new BufferedWriter(new FileWriter(fileTo));
             writer.write(textEncoded);
             writer.close();
             
	        System.out.println("Done! "+fileTo);
            
    	}catch(Exception ex){
 	       ex.printStackTrace();
     	}
    	
    }
    
    public String getStringFromDocument(Document doc){
    	try{
	    	TransformerFactory tf = TransformerFactory.newInstance();
	    	Transformer trans = tf.newTransformer();
	    	StringWriter sw = new StringWriter();
	    	trans.transform(new DOMSource(doc), new StreamResult(sw));
	    	return sw.toString();
    	}catch(Exception ex){
	       ex.printStackTrace();
	       return null;
    	}
    }
	
    public void convert(String fileFROM,String fileTO,String fechaHoyXML){
    	try {
    		
    		File xmlFile = new File(fileFROM);
    		
    		InputStreamReader r = new InputStreamReader(new FileInputStream(xmlFile));
    		
    		String content = new String(Files.readAllBytes(Paths.get(fileFROM)), r.getEncoding());
    		String content1=new String();
    		String content2=new String();
    		String content3=new String();
    		
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc = documentBuilder.parse(xmlFile);
            doc.getDocumentElement().normalize();
            
            Boolean taxInclusiveAmount=haveTaxInclusiveAmount(doc);
            if(!taxInclusiveAmount){
            	//addTaxInclusiveAmount(doc);
            	String sTaxInclusiveAmount=getPriceAmountFromAlternativeConditionPrice(doc);
            	content1=content.replaceFirst("<cbc:ChargeTotalAmount currencyID=\"PEN\">", "<cbc:TaxInclusiveAmount currencyID=\"PEN\">"+sTaxInclusiveAmount+"</cbc:TaxInclusiveAmount>\n<cbc:ChargeTotalAmount currencyID=\"PEN\">");
            }else{
            	content1=content;
            }
            
            Boolean paymentTerms=havePaymentTerms(doc);
            if(!paymentTerms){
            	//addPaymentTerms(doc);
        		content2=content1.replaceFirst("<cac:TaxTotal>", "<cac:PaymentTerms><cbc:ID>FormaPago</cbc:ID><cbc:PaymentMeansID>Contado</cbc:PaymentMeansID></cac:PaymentTerms>\n<cac:TaxTotal>\n");
            }else{
            	content2=content1;
            }
            
            Boolean allowanceCharge=haveAllowanceCharge(doc);
            if(!allowanceCharge){
            	//addAllowanceCharge(doc);
            	String sChargeTotalAmount=getChargeTotalAmount(doc);
            	content3=content2.replaceFirst("<cac:TaxTotal>", "<cac:AllowanceCharge><cbc:ChargeIndicator>true</cbc:ChargeIndicator><cbc:AllowanceChargeReasonCode listAgencyName=\"PE:SUNAT\" listName=\"Cargo/descuento\" listURI=\"urn:pe:gob:sunat:cpe:see:gem:catalogos:catalogo53\">46</cbc:AllowanceChargeReasonCode><cbc:MultiplierFactorNumeric>1</cbc:MultiplierFactorNumeric><cbc:Amount currencyID=\"PEN\">"+sChargeTotalAmount+"</cbc:Amount><cbc:BaseAmount currencyID=\"PEN\">"+sChargeTotalAmount+"</cbc:BaseAmount></cac:AllowanceCharge>\n<cac:TaxTotal>\n");
            }else{
            	content3=content2;
            }

            //fuerza por el cambio de fecha del comprobante
            
            String sIssueDate=getIssueDate(doc);
            if(sIssueDate!=null&&sIssueDate.trim().length()>0){
            	content3=content3.replaceAll("<cbc:IssueDate>"+sIssueDate+"</cbc:IssueDate>", "<cbc:IssueDate>"+fechaHoyXML+"</cbc:IssueDate>");
            }
            
            //System.out.println("to! "+fileTO);
            //System.out.println(content3);
            
            BufferedWriter writer = new BufferedWriter(new FileWriter(fileTO));
            writer.write(content3);
            writer.close();
            
            System.out.println("Done! "+fileTO);
            
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    
    public Document convertDescuento(String fileFROM){
    	Document doc = null;
    	try {
    		File xmlFile = new File(fileFROM);
    		
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            doc = documentBuilder.parse(xmlFile);
            doc.getDocumentElement().normalize();
            
            Boolean allowanceTotalAmount=haveAllowanceTotalAmount(doc);
            if(allowanceTotalAmount){
            	fixAllowanceInvoiceLine(doc);
            	addPaymentTerms(doc);
            	addAllowanceCharge(doc);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    	
    	return doc;
    }
    
    public void addAllowanceCharge(Document doc){
		
		String sChargeTotalAmount=getChargeTotalAmount(doc);
		
		Element AllowanceCharge = doc.createElement("cac:AllowanceCharge");
		
			Element ChargeIndicator = doc.createElement("cbc:ChargeIndicator");
			AllowanceCharge.appendChild(ChargeIndicator);
			ChargeIndicator.appendChild(doc.createTextNode("true"));
			
			Element AllowanceChargeReasonCode = doc.createElement("cbc:AllowanceChargeReasonCode");
			AllowanceCharge.appendChild(AllowanceChargeReasonCode);
			AllowanceChargeReasonCode.appendChild(doc.createTextNode("46"));
			AllowanceChargeReasonCode.setAttribute("listAgencyName", "PE:SUNAT");
			AllowanceChargeReasonCode.setAttribute("listName", "Cargo/descuento");
			AllowanceChargeReasonCode.setAttribute("listURI", "urn:pe:gob:sunat:cpe:see:gem:catalogos:catalogo53");
			
			Element MultiplierFactorNumeric = doc.createElement("cbc:MultiplierFactorNumeric");
			AllowanceCharge.appendChild(MultiplierFactorNumeric);
			MultiplierFactorNumeric.appendChild(doc.createTextNode("1"));
			
			Element Amount = doc.createElement("cbc:Amount");
			AllowanceCharge.appendChild(Amount);
			Amount.appendChild(doc.createTextNode(sChargeTotalAmount));/********************************/
			Amount.setAttribute("currencyID", "PEN");
			
			Element BaseAmount = doc.createElement("cbc:BaseAmount");
			AllowanceCharge.appendChild(BaseAmount);
			BaseAmount.appendChild(doc.createTextNode(sChargeTotalAmount));/********************************/
			BaseAmount.setAttribute("currencyID", "PEN");
		
		Node Invoice = doc.getElementsByTagName("Invoice").item(0);
		Invoice.insertBefore(AllowanceCharge, getNode("cac:TaxTotal",Invoice.getChildNodes()));
	}
    
    public void addPaymentTerms(Document doc){
		Element PaymentTerms = doc.createElement("cac:PaymentTerms");
			Element ID = doc.createElement("cbc:ID");
			PaymentTerms.appendChild(ID);
			ID.appendChild(doc.createTextNode("FormaPago"));
			
			Element PaymentMeansID = doc.createElement("cbc:PaymentMeansID");
			PaymentTerms.appendChild(PaymentMeansID);
			PaymentMeansID.appendChild(doc.createTextNode("Contado"));
		
		Node Invoice = doc.getElementsByTagName("Invoice").item(0);
		Invoice.insertBefore(PaymentTerms, getNode("cac:TaxTotal",Invoice.getChildNodes()));
	}

    public void removeAllowanceCharge(Document doc){
    	 Node Invoice = doc.getElementsByTagName("Invoice").item(0);
   	 
	   	 NodeList targets = doc.getElementsByTagName("cac:AllowanceCharge");
	   	 int toDelete = targets.getLength();
	   	 for(int i = 0; i < toDelete; i++){
	   		 Invoice.removeChild(targets.item(i));
	   	 }
    }
    
    
    
    public void fixAllowanceInvoiceLine(Document doc){
    	
    	Double multiplierAllowance=getMultiplierFactorNumeric(doc);
    	
    	removeAllowanceCharge(doc);
    	
    	NodeList listInvoiceLine = doc.getElementsByTagName("cac:InvoiceLine");
    	for(int i=0;i<listInvoiceLine.getLength();i++){
    		Node InvoiceLine = listInvoiceLine.item(i);
   		 	if(InvoiceLine!=null){
   		 		
   		 		Double PriceAmount=0.0;
   		 		Double TaxPercent=0.0;	
   		 	
   		 		Node Price= getNode("cac:Price",InvoiceLine.getChildNodes());
   		 		if(Price!=null){
   		 			PriceAmount=getPriceAmount(Price);	
   		 		}
	   	
   		 		Node TaxTotal= getNode("cac:TaxTotal",InvoiceLine.getChildNodes());
		 		if(TaxTotal!=null){
		 			TaxPercent=getTaxPercent(TaxTotal);	
		 		}
   		 			
		 		/**Fix Allowance Invoice Line*/
		 		Double Allowance=PriceAmount*multiplierAllowance;
		 		Double LineAmount=PriceAmount-Allowance;
		 		Double TaxAmount=LineAmount*TaxPercent/100;
		 		Double PriceReference=LineAmount+TaxAmount;
		 		/**Fix Allowance Invoice Line end*/
		 		
		 		//<cbc:LineExtensionAmount
		 		Node LineExtensionAmount= getNode("cbc:LineExtensionAmount",InvoiceLine.getChildNodes());
		 		if(LineExtensionAmount!=null){
		 			LineExtensionAmount.setTextContent(Util.format(LineAmount));
		 		}
	   		 	
		 		//<cac:PricingReference>
		 		Node PricingReference= getNode("cac:PricingReference",InvoiceLine.getChildNodes());
		 		if(PricingReference!=null){
		 			NodeList lChild = PricingReference.getChildNodes();
	     		    for(int n=0;n<lChild.getLength();n++){
	     	     		Node child = lChild.item(n);
	     	     		if(child.getNodeName().equals("cac:AlternativeConditionPrice")){
	     	     			
	     	     			NodeList lChild2 = child.getChildNodes();
	     	     		    for(int m=0;m<lChild2.getLength();m++){
	     	     	     		Node child2 = lChild2.item(m);
	     	     	     		if(child2.getNodeName().equals("cbc:PriceAmount")){
	     	     	     			child2.setTextContent(Util.format(PriceReference));
	     	     	     			break;
	     	     	     		}
	     	     	     	}
	     	     		    break;
	     	     		}
	     		    }
		 		}
		 		
		 		//<cac:AllowanceCharge><!--ADD-->
		 		Element AllowanceCharge= doc.createElement("cac:AllowanceCharge");
		 		//InvoiceLine.appendChild(AllowanceCharge);
		 		InvoiceLine.insertBefore(AllowanceCharge, getNode("cac:TaxTotal",InvoiceLine.getChildNodes()));
		 		
		 			Element ChargeIndicator= doc.createElement("cbc:ChargeIndicator");
			 		AllowanceCharge.appendChild(ChargeIndicator);
			 		ChargeIndicator.setTextContent("false");
			 		Element Amount= doc.createElement("cbc:Amount");
			 		AllowanceCharge.appendChild(Amount);
			 		Amount.setTextContent(Util.format(Allowance));
			 		Amount.setAttribute("currencyID", "PEN");
			 		
		 		
		 		
		 		//<cac:TaxTotal>
		 		Node NodeTaxTotal= getNode("cac:TaxTotal",InvoiceLine.getChildNodes());
		 		if(NodeTaxTotal!=null){
		 			NodeList lChild = NodeTaxTotal.getChildNodes();
	     		    for(int n=0;n<lChild.getLength();n++){
	     	     		Node child = lChild.item(n);
	     	     		if(child.getNodeName().equals("cbc:TaxAmount")){
	     	     			child.setTextContent(Util.format(TaxAmount));
	     	     		}else if(child.getNodeName().equals("cac:TaxSubtotal")){
	     	     			NodeList lChild2 = child.getChildNodes();
	    	     		    for(int k=0;k<lChild2.getLength();k++){
	    	     	     		Node child2 = lChild2.item(k);
	    	     	     		if(child2.getNodeName().equals("cbc:TaxableAmount")){
	    	     	     			child2.setTextContent(Util.format(LineAmount));
	    	     	     		}else if(child2.getNodeName().equals("cbc:TaxAmount")){
	    	     	     			child2.setTextContent(Util.format(TaxAmount));
	    	     	     		}
	    	     		    }
	     	     		}
	     		    }
		 		}
   		 	}
    	}
    }
    
    public Double getTaxPercent(Node TaxTotal){
    	Double Percent=0.0;
    	
    	NodeList lChildTaxTotal = TaxTotal.getChildNodes();
	    for(int j=0;j<lChildTaxTotal.getLength();j++){
     		Node childNode = lChildTaxTotal.item(j);
     		if(childNode.getNodeName().equals("cac:TaxSubtotal")){
     			
     			NodeList lChildTaxSubtotal = childNode.getChildNodes();
     		    for(int n=0;n<lChildTaxSubtotal.getLength();n++){
     	     		Node childTaxSubtotal = lChildTaxSubtotal.item(n);
     	     		if(childTaxSubtotal.getNodeName().equals("cac:TaxCategory")){
     	     			
     	     			NodeList lChildTaxCategory = childTaxSubtotal.getChildNodes();
     	     		    for(int m=0;m<lChildTaxCategory.getLength();m++){
     	     	     		Node childTaxCategory = lChildTaxCategory.item(m);
     	     	     		if(childTaxCategory.getNodeName().equals("cbc:Percent")){
     	     	     			String txtPercent=childTaxCategory.getTextContent();
     	     	     			Percent=Util.stringToDouble(txtPercent);
     	     	     			break;
     	     	     		}
     	     	     	}
     	     		    break;
     	     		}
     	     	}
     			break;
     		}
     	}
	    return Percent;
    }
    
    public Double getPriceAmount(Node Price){
    	Double PriceAmount=0.0;
    	NodeList lChildPrice = Price.getChildNodes();
	    for(int j=0;j<lChildPrice.getLength();j++){
     		Node childNode = lChildPrice.item(j);
     		if(childNode.getNodeName().equals("cbc:PriceAmount")){
     			String txtPriceAmount=childNode.getTextContent();
     			PriceAmount=Util.stringToDouble(txtPriceAmount);
     			break;//ok
     		}
     	}
	    return PriceAmount;
    }
    
    public Node getNode(String tagNode,NodeList listNode){
    	for(int i=0;i<listNode.getLength();i++){
    		 Node child = listNode.item(i);
    		 if(child!=null){
	     		if(child.getNodeName().equals(tagNode)){
	     			return child;
	     		}
    		 }
		}
    	return null;
    }
    

    public Double getMultiplierFactorNumeric(Document doc){
    	Double multiplierFactorNumeric=0.0;
    	Node allowance=getAllowanceNode(doc);
    	if(allowance!=null){
    		NodeList nodes = allowance.getChildNodes();
    	    for(int j=0;j<nodes.getLength();j++){
         		Node childNode = nodes.item(j);
         		if(childNode.getNodeName().equals("cbc:MultiplierFactorNumeric")){
         			String txtMultiplierFactorNumeric=childNode.getTextContent();
         			multiplierFactorNumeric=Util.stringToDouble(txtMultiplierFactorNumeric);
         			break;//ok
         		}
         	}
    	}
    	return multiplierFactorNumeric;
    }
    
    public Node getAllowanceNode(Document doc){
    	//ok
    	Node allowance=null;
    	
    	NodeList lAllowanceCharge = doc.getElementsByTagName("cac:AllowanceCharge");
    	for(int i=0;i<lAllowanceCharge.getLength();i++){
    		 Node allowanceCharge = lAllowanceCharge.item(i);
    		 if(allowanceCharge!=null){
    			NodeList nodes = allowanceCharge.getChildNodes();
    		    for(int j=0;j<nodes.getLength();j++){
		     		Node childNode = nodes.item(j);
		     		if(childNode.getNodeName().equals("cbc:ChargeIndicator")){
		     			String txtChargeIndicator=childNode.getTextContent();
		     			Boolean chargeIndicator=Util.stringToBolean(txtChargeIndicator);
		     			if(chargeIndicator==Boolean.FALSE){//Allowance
		     				allowance=allowanceCharge;
		     				break;
		     			}
		     		}
		     	}
    		 }
    	}
    	
    	return allowance;
    }
    
    public Boolean haveAllowanceTotalAmount(Document doc){
	   	 Node allowanceTotalAmount = doc.getElementsByTagName("cbc:AllowanceTotalAmount").item(0);
	   	 if(allowanceTotalAmount!=null){
	   		
	   		String totalAmount=allowanceTotalAmount.getTextContent();
	   		Double dTotalAmount=Util.stringToDouble(totalAmount);
	   		if(dTotalAmount>0)
	   			return Boolean.TRUE;
	   	 }
	   	 return Boolean.FALSE;
   }
    
    public String getIssueDate(Document doc){
		String dateInvoice="";
    	
    	NodeList lIssueDate = doc.getElementsByTagName("cbc:IssueDate");
    	for(int i=0;i<lIssueDate.getLength();i++){
    		 Node issueDate = lIssueDate.item(i);
    		 dateInvoice=issueDate.getTextContent();
    	}
    	
    	return dateInvoice;
	}
    
    public Boolean havePaymentTerms(Document doc){
	   	 Node lPaymentTerms = doc.getElementsByTagName("cac:PaymentTerms").item(0);
	   	 if(lPaymentTerms!=null){
	   		return Boolean.TRUE;
	   	 }
	   	 return Boolean.FALSE;
    }
    
    public Boolean haveTaxInclusiveAmount(Document doc){
    	 Node legalMonetaryTotal = doc.getElementsByTagName("cac:LegalMonetaryTotal").item(0);
		 NodeList nodes = legalMonetaryTotal.getChildNodes();
	     if (nodes != null) {
	     	for(int i=0;i<nodes.getLength();i++){
	     		Node childNode = nodes.item(i);
	     		if(childNode.getNodeName().equals("cbc:TaxInclusiveAmount")){
	     			return Boolean.TRUE;
	     		}
	     	}
	     }
    	 return Boolean.FALSE;
    }
    
    public Boolean haveAllowanceCharge(Document doc){
    	 Node lPaymentTerms = doc.getElementsByTagName("cac:AllowanceCharge").item(0);
	   	 if(lPaymentTerms!=null){
	   		return Boolean.TRUE;
	   	 }
	   	 return Boolean.FALSE;
    }
	
    
    
    public void addTaxInclusiveAmount(Document doc){
    	
    	//<cac:PaymentTerms>
    	//<cbc:ID>FormaPago</cbc:ID>
    	//<cbc:PaymentMeansID>Contado</cbc:PaymentMeansID>
    	//</cac:PaymentTerms>
    	
    	Node legalMonetaryTotal = doc.getElementsByTagName("cac:LegalMonetaryTotal").item(0);
    	if(legalMonetaryTotal!=null){
    		String sTaxInclusiveAmount=getPriceAmountFromAlternativeConditionPrice(doc);
    		
    		Element addNode = doc.createElement("cbc:TaxInclusiveAmount");
        	legalMonetaryTotal.appendChild(addNode);
        	addNode.appendChild(doc.createTextNode(sTaxInclusiveAmount));
        	addNode.setAttribute("currencyID", "PEN");	
        	
    	}
    }

    public String getPriceAmountFromAlternativeConditionPrice(Document doc){
    	
    	Double TaxInclusiveAmount=0.0;
    	
    	NodeList listInvoiceLine = doc.getElementsByTagName("cac:InvoiceLine");
    	for(int i=0;i<listInvoiceLine.getLength();i++){
    		
    		Double PriceAmount=0.0;
    		Double InvoicedQuantity=0.0;
    		
    		 Node invoiceLine = listInvoiceLine.item(i);
    		
    		 NodeList nodes = invoiceLine.getChildNodes();
    	     if (nodes != null) {
    	     	for(int j=0;j<nodes.getLength();j++){
    	     		Node childNode = nodes.item(j);
    	     		if(childNode.getNodeName().equals("cac:PricingReference")){
    	     			
    	     			NodeList nodesPricingReference = childNode.getChildNodes();
        	            if (nodesPricingReference != null) {
        	            	for(int k=0;k<nodesPricingReference.getLength();k++){
        	            		Node childNode2 = nodesPricingReference.item(k);
        	            		if(childNode2.getNodeName().equals("cac:AlternativeConditionPrice")){
        	            			
        	            			NodeList nodesAlternativeConditionPrice = childNode2.getChildNodes();
        	        	            if (nodesAlternativeConditionPrice != null) {
        	        	            	for(int g=0;g<nodesAlternativeConditionPrice.getLength();g++){
        	        	            		Node childNode3 = nodesAlternativeConditionPrice.item(g);
        	        	            		if(childNode3.getNodeName().equals("cbc:PriceAmount")){
        	        	            			String sPriceAmount=childNode3.getTextContent();
        	        	            			PriceAmount=Util.stringToDouble(sPriceAmount);
        	        	            		}
        	        	            	}
        	        	            }
        	        	            
        	            		}
        	            	}
        	            }
    	     		}else if(childNode.getNodeName().equals("cbc:InvoicedQuantity")){
    	     			String sInvoicedQuantity=childNode.getTextContent();
    	     			InvoicedQuantity=Util.stringToDouble(sInvoicedQuantity);
    	     		}
    	     	}
    	     }
    	     
    	     TaxInclusiveAmount=TaxInclusiveAmount+PriceAmount*InvoicedQuantity;
    	}
    	
    	return Util.format(TaxInclusiveAmount);
    }
    
	
    
	public String getChargeTotalAmount(Document doc){
		Double ChargeTotalAmount=0.0;
    	
    	NodeList LegalMonetaryTotal = doc.getElementsByTagName("cac:LegalMonetaryTotal");
    	for(int i=0;i<LegalMonetaryTotal.getLength();i++){
    		
    		 Node legalMonetaryTotal = LegalMonetaryTotal.item(i);
    		
    		 NodeList nodes = legalMonetaryTotal.getChildNodes();
    	     if (nodes != null) {
    	     	for(int j=0;j<nodes.getLength();j++){
    	     		Node childNode = nodes.item(j);
    	     		if(childNode.getNodeName().equals("cbc:ChargeTotalAmount")){
    	     			String sChargeTotalAmount=childNode.getTextContent();
    	     			ChargeTotalAmount=Util.stringToDouble(sChargeTotalAmount);
    	     		}
    	     	}
    	     }
    	}
    	
    	return Util.format(ChargeTotalAmount);
	}
}
